"""Custom widgets for Edda Workflow Manager."""

from edda_workflow_manager.widgets.stats_card import StatsCard
from edda_workflow_manager.widgets.filter_bar import FilterBar

__all__ = [
    "StatsCard",
    "FilterBar",
]
